function map = DEM_main(v,f,population,initial_map,epsilon,step_ratio)
% Compute the density-equalizing map of a simply-connected open surface based on a prescribed population (with free boundary condition)
%
% Input:
% v: nv x 2 or nv x 3 vertex coordinates
% f: nf x 3 triangulation
% population: nf x 1 positive array
% initial_map: nv x 2 coordinates of the initial flattening map
% (Optional) epsilon: stopping parameter (default = 0.001)
% (Optional) step_ratio: rescale the step size (default = 1)

% Output:
% map: nv x 2 density-equalizing flattening map 

% Remark: 
% The area of the result will be the same as that of the input mesh.
%
% If you use this code in your own work, please cite the following paper:
% [1] G. P. T. Choi and C. H. Rycroft, 
%     "Density-equalizing maps for simply connected open surfaces."
%     SIAM Journal on Imaging Sciences, 11(2), pp. 1134-1178, 2018.
% 
% Copyright (c) 2017-2018, Gary Pui-Tung Choi
% https://scholar.harvard.edu/choi

% default stopping parameter
if ~exist('epsilon','var')
    epsilon = 1e-3;
end

% rescale the step size by a multiple
if ~exist('step_ratio','var')
    step_ratio = 1;
end

% compute the number of faces and vertices
nf = length(f);
nv = length(v);

area_original = sum(face_area(f,v));
population = population*(area_original/sum(population)); 

% construct sea
[r,f_with_sea] = construct_sea(initial_map,f); % construct sea using reflection
r = r * sqrt(area_original / sum(face_area(f_with_sea(1:nf,:),r(1:nv,:)))); % restore the size

% density
rho_f = population ./ face_area(f_with_sea(1:nf,:),r(1:nv,:)); 
rho_v = zeros(length(r),1);
rho_v(1:nv) = f2v_area(r(1:nv,:),f_with_sea(1:nf,:))*rho_f; % interpolate rho at vertices
rho_v(nv+1:end) = mean(rho_v(1:nv)); % define density at the sea

dt = min([min(rho_v(1:nv))/mean(rho_v(1:nv)),mean(rho_v(1:nv))/max(rho_v(1:nv))])*area_original*step_ratio;

step = 1;
disp('Step     std(rho)/mean(rho)');  
disp([num2str(step), '        ',num2str(std(rho_v(1:nv))/mean(rho_v(1:nv)))]);

rho_v_error = zeros(1,50);
rho_v_error(1) = std(rho_v(1:nv))/mean(rho_v(1:nv));

while rho_v_error(step) > epsilon
    % update rho 
    L = laplace_beltrami(r,f_with_sea);
    A = lumped_mass_matrix(r,f_with_sea);
    rho_v_temp = (A - dt*L) \ (A*rho_v);
    
    if sum(sum(isnan(rho_v_temp)))~=0
        break;
    end

    % update gradient
    grad_rho_temp_f = compute_gradient(r,f_with_sea,rho_v_temp);
    grad_rho_temp_v = f2v_area(r,f_with_sea)*grad_rho_temp_f;

    % update displacement
    dr = [grad_rho_temp_v(:,1)./rho_v_temp, grad_rho_temp_v(:,2)./rho_v_temp];
    r_temp = r - dr*dt;

    % update other parameters
    r = r_temp; 
    rho_v = rho_v_temp;
    step = step + 1;
    
    rho_v_error(step) = std(rho_v(1:nv))/mean(rho_v(1:nv));
    disp([num2str(step), '        ',num2str(std(rho_v(1:nv))/mean(rho_v(1:nv)))]);
    
    if step >= 50
        break;
    end
    
end

% finally normalize the area
r = r * sqrt(area_original / sum(face_area(f_with_sea(1:nf,:),r(1:nv,:))));
map = r(1:nv,1:2);

fprintf('Completed.\n');
