function map = DEM_rect(v,f,population,initial_map,corner,epsilon,step_ratio)
% Compute the density-equalizing map of a simply-connected open surface based on a prescribed population (with square/rectangular boundary condition)
%
% Input:
% v: nv x 2 or nv x 3 vertex coordinates
% f: nf x 3 triangulation
% population: nf x 1 positive array
% initial_map: nv x 2 coordinates of the initial flattening map
% corner: 1 x 4 vertex indices of the corner
% (Optional) epsilon: stopping parameter (default = 0.001)
% (Optional) step_ratio: rescale the step size (default = 1)
%
% Output:
% map: m x 2 density-equalizing map 
%
% Remark: 
% The area of the result will be the same as that of the input mesh.
%
% If you use this code in your own work, please cite the following paper:
% [1] G. P. T. Choi and C. H. Rycroft, 
%     "Density-equalizing maps for simply connected open surfaces."
%     SIAM Journal on Imaging Sciences, 11(2), pp. 1134-1178, 2018.
% 
% Copyright (c) 2017-2018, Gary Pui-Tung Choi
% https://scholar.harvard.edu/choi

% default stopping parameter
if ~exist('epsilon','var')
    epsilon = 1e-3;
end

% rescale the step size by a multiple
if ~exist('step_ratio','var')
    step_ratio = 1;
end

% find boundary
TR = TriRep(f,v); 
bdy_e = freeBoundary(TR); 
bdy_v = bdy_e(:,1); 
bdy_f = zeros(length(bdy_e),1);
for i = 1:length(bdy_e)
    bdy_f(i) = find(sum((f == bdy_e(i,1)) + (f==bdy_e(i,2)),2) == 2);
end

area_original = sum(face_area(f,v));

r = initial_map(:,1:2);
r = r * sqrt(area_original / sum(face_area(f,r)));

r_bdy = r(bdy_v,:);

% density
rho_f = population ./ face_area(f,r); 
rho_v = f2v_area(r,f)*rho_f; 

step = 1;

dt = min([min(rho_v)/mean(rho_v),mean(rho_v)/max(rho_v)])*area_original*step_ratio;

disp('Step     std(rho)/mean(rho)');  
disp([num2str(step), '        ',num2str(std(rho_v)/mean(rho_v))]);
    
rho_v_error = zeros(1,50);
rho_v_error(step) = std(rho_v)/mean(rho_v);


while rho_v_error(step) > epsilon
    % update rho 
    L = laplace_beltrami(r,f);
    A = lumped_mass_matrix(r,f);
    rho_v_temp = (A - dt*L) \ (A*rho_v);
    
    if sum(sum(isnan(rho_v_temp)))~=0
        break;
    end

    % update gradient
    grad_rho_temp_f = compute_gradient(r,f,rho_v_temp);
    grad_rho_temp_v = f2v_area(r,f)*grad_rho_temp_f;
    
    % projection 
    bdy_normal1 = [-(r(bdy_e(:,1),2) - r(bdy_e(:,2),2)),  (r(bdy_e(:,1),1) - r(bdy_e(:,2),1))];
    bdy_normal1 = [bdy_normal1(:,1) ./ sqrt(bdy_normal1(:,1).^2 + bdy_normal1(:,2).^2) , bdy_normal1(:,2) ./ sqrt(bdy_normal1(:,1).^2 + bdy_normal1(:,2).^2)];
    bdy_normal2 = [-(r(bdy_e([end,1:end-1],1),2) - r(bdy_e([end,1:end-1],2),2)),  (r(bdy_e([end,1:end-1],1),1) - r(bdy_e([end,1:end-1],2),1))];
    bdy_normal2 = [bdy_normal2(:,1) ./ sqrt(bdy_normal2(:,1).^2 + bdy_normal2(:,2).^2) , bdy_normal2(:,2) ./ sqrt(bdy_normal2(:,1).^2 + bdy_normal2(:,2).^2)];
    bdy_normal = ( bdy_normal1 + bdy_normal2 )/2;
    grad_rho_temp_v(bdy_v,:) = grad_rho_temp_v(bdy_v,:) - ...
        [sum(bdy_normal.*grad_rho_temp_v(bdy_v,:),2).*bdy_normal(:,1), ...
        sum(bdy_normal.*grad_rho_temp_v(bdy_v,:),2).*bdy_normal(:,2)];
    
    % enforce corner position
    grad_rho_temp_v(corner,:) = 0;
    
    % update displacement 
    dr = [grad_rho_temp_v(:,1) ./ rho_v_temp, grad_rho_temp_v(:,2) ./ rho_v_temp];
    r_temp = r - dr*dt;
    r = r_temp; 
    
    % update other parameters 
    rho_v = rho_v_temp;
    step = step + 1;

    rho_v_error(step) = std(rho_v)/mean(rho_v);
    disp([num2str(step), '        ',num2str(std(rho_v)/mean(rho_v))]);

    if step > 50
        break;
    end
end

% a final projection to correct any boundary shape mismatch due to numerical approximation error
proj_bdy = distance2curve(r_bdy,r(bdy_v,:));
r(bdy_v,:) = proj_bdy;

% finally normalize the area
map = r * sqrt(area_original / sum(face_area(f,r)));
fprintf('Completed.\n');