function initial_map = construct_initial_map(v,f,flatten_method)
% Compute curvature-based initial map using Tutte/locally authalic method
%
% If you use this code in your own work, please cite the following paper:
% [1] G. P. T. Choi and C. H. Rycroft, 
%     "Density-equalizing maps for simply connected open surfaces."
%     SIAM Journal on Imaging Sciences, 11(2), pp. 1134-1178, 2018.
% 
% Copyright (c) 2017-2018, Gary Pui-Tung Choi
% https://scholar.harvard.edu/choi

% currently only use nonnegative curvature to construct a convex plane curve
% may need modification in case negative curvature is desired

if size(v,2) == 2
    initial_map = v;
else
    % for surface flattening, 1 = Tutte, 2 = locally authalic
    if ~exist('flatten_method','var')
        flatten_method = 1;
    end
    
    TR = TriRep(f,v); B = freeBoundary(TR); bdyid = B(:,1); % find bdy
    % flatten the boundary based on curvature
    bdy = curvature_based_flattening(v(bdyid,:));
        
    if flatten_method == 1
        % tutte map
        initial_map = tutte_map(v,f,bdyid,bdy);
    else
        % locally authalic map
        initial_map = locally_authalic_map(v,f,bdyid,bdy);
        
        % check bijectivity
        a = initial_map(f(:,2),:) - initial_map(f(:,1),:);
        b = initial_map(f(:,3),:) - initial_map(f(:,1),:);
        normal_direction = ((a(:,1).*b(:,2) - a(:,2).*b(:,1)) > 0);
        if length(unique(normal_direction)) > 1
           % contain overlaps, switch to tutte map
           initial_map = tutte_map(v,f,bdyid,bdy);
        end
    end
end
