function [v_with_sea,f_with_sea] = construct_sea(v,f)
% Construct a sea surrounding the planar mesh.
% 
% Input:
% v: n x 2
% f: m x 3
%
% Output: 
% v_with_sea: N x 2
% f_with_sea M x 3
%
% If you use this code in your own work, please cite the following paper:
% [1] G. P. T. Choi and C. H. Rycroft, 
%     "Density-equalizing maps for simply connected open surfaces."
%     SIAM Journal on Imaging Sciences, 11(2), pp. 1134-1178, 2018.
% 
% Copyright (c) 2017-2018, Gary Pui-Tung Choi
% https://scholar.harvard.edu/choi

TR = TriRep(f,v); B = freeBoundary(TR); bd = B(:,1);

z = complex(v(:,1), v(:,2));
z = z - mean(z); % translate center to 0
z = z / max(abs(z)); % shrink it into unit disk

z_bdy = z(bd);

% compute average bdy edge length
edge_size = mean(abs(z_bdy - z_bdy([2:end,1])));

% shrink a bit
z = z * (1 - 3*edge_size);
z_bdy = z(bd);

% make unit circle
circle = exp(1i*(0.01:edge_size:2*pi))';

% generate interior points
[X,Y] = meshgrid(-1.1:edge_size:1.1, -1.1:edge_size:1.1); 
X(2:2:end,:)=X(2:2:end,:)+edge_size/2; 
X = X(:); Y = Y(:);

insidez = find(InPolygon(X,Y,(1+edge_size/2)*real(z_bdy),(1+edge_size/2)*imag(z_bdy)));
insidec = find(InPolygon(X,Y,(1-edge_size/2)*real(circle),(1-edge_size/2)*imag(circle)));

newv = setdiff(insidec,insidez); 
new_vertices = [X(newv), Y(newv)];

% triangulate
v_disk = [real(z),imag(z);new_vertices;real(circle),imag(circle)];

if exist('delaunayTriangulation','builtin')
    DT = delaunayTriangulation(v_disk,[bd,bd([2:end,1])]);
    f_new = (DT.ConnectivityList);
else DT = DelaunayTri(v_disk,[bd,bd([2:end,1])]);
    f_new = DT.Triangulation;
end

% clean triangles inside the original boundary
centroid = (v_disk(f_new(:,1),:) + v_disk(f_new(:,2),:) + v_disk(f_new(:,3),:))/3;
IN2 = InPolygon(centroid(:,1), centroid(:,2), real(z_bdy), imag(z_bdy));

f_new(IN2,:) = [];
f_disk = [f;f_new]; % replace the interior by the given triangulation

% reflection to make a sea
z_new = complex(v_disk(:,1), v_disk(:,2));
centroid_new = (z_new(f_disk(:,1),:) + z_new(f_disk(:,2),:) + z_new(f_disk(:,3),:))/3;
reflect_index_f = (abs(1./conj(centroid_new)) <= 5); % should reflect these triangles
f_temp = f_disk(reflect_index_f,:);
reflect_index_v = setdiff(unique(f_temp(:)),(length(z) + length(new_vertices) +1):length(v_disk)) ; % should reflect these vertices
ignored = length(v_disk) - length(reflect_index_v) - length((length(z) + length(new_vertices) +1):length(v_disk));
[~,~,IC] = unique(f_temp(:));
f_temp(IC<= length(z) + length(new_vertices) - ignored) = IC(IC <= length(z) + length(new_vertices) - ignored);
f_temp(IC<= length(z) + length(new_vertices) - ignored) = f_temp(IC<= length(z) + length(new_vertices) - ignored) + length(v_disk);
z_reflected = [z_new;1./conj(z_new(reflect_index_v))]; 

% glue the result
f_with_sea = [f_disk;fliplr(f_temp)];
v_with_sea = [real(z_reflected), imag(z_reflected)];

