function bdy = curvature_based_flattening(v)
% Flatten a 3D curve onto the plane based on curvature.
%
% Input: 
% v: nx3 ordered vertices
%
% Output: 
% bdy: nx1 complex coordinates of a closed plane curve
%
% If you use this code in your own work, please cite the following paper:
% [1] G. P. T. Choi and C. H. Rycroft, 
%     "Density-equalizing maps for simply connected open surfaces."
%     SIAM Journal on Imaging Sciences, 11(2), pp. 1134-1178, 2018.
% 
% Copyright (c) 2017-2018, Gary Pui-Tung Choi
% https://scholar.harvard.edu/choi

if size(v,1) < size(v,2)
    v = v';
end

n = length(v);
ds = sqrt(sum((v([1:n,1],:) - v([2:n,1:2],:)).^2,2));
ds = (ds + ds([2:end,1]))/2;

% compute the curvature
k = compute_curve_curvature(v([n;(1:n)';(1:3)'],:)); 

% enforce int k(s) ds = 2 pi
k = k / sum(ds(1:n).*k(1:n)) * 2*pi; 

s = zeros(n,1);
for i = 2:n+1
    s(i) = s(i-1) + ds(i-1);
end

% theta(t) = int^t kappa(u) du
theta = zeros(n,1);
for i = 2:n+1
    theta(i) = trapz(s(1:i),k(1:i));
end

% phi(s) = (int^s cos theta(t) dt, int^s sin theta(t) dt)
x = zeros(n,1); y = zeros(n,1);
for i = 2:n+1
    x(i) = trapz(s(1:i),cos(theta(1:i)));
    y(i) = trapz(s(1:i),sin(theta(1:i))); 
end

% close the loop
x = x - (s/sum(ds))*(x(end) - x(1));
y = y - (s/sum(ds))*(y(end) - y(1));

x = x((1:n)');
y = y((1:n)');
bdy = complex(x - mean(x),y - mean(y));